/***********************************************//** @file HepMCVisualFilter.cxx
* 
* Implementation of the filter and helper class
*
* Visual filter is a filter class with GUI window for HepMC GenEvents
* Basic functions include: Vertex finding, particle and vertex slimming.
*
* @author <A HREF="mailto:sboeser@hep.ucl.ac.uk">Sebastian B&ouml;ser</A>
* @date 06.08.07
*
*//****************************************************************************/

   
#include "HepMCVisual/HepMCVisualFilter.h"

//we have added our classes to namespace HepMC
using namespace HepMC;

/*************************/
/*      VisualFilter     */
/*************************/
  
//default constructor
VisualFilter::VisualFilter() {

  //nothing to be done so far
}


//default constructor
VisualFilter::~VisualFilter() {

  //nothing to be done so far
}

Bool_t VisualFilter::FindSignalProcess( HepMC::GenEvent& event,  
                                       const std::vector<Int_t> in,
                                       const std::vector<Int_t> out ){

  //Loop over all vertices 
  for (GenEvent::vertex_iterator ver = event.vertices_begin();
       ver != event.vertices_end(); ver++){
    
    //Make sure vertex has enough in and out particles at all
    if ((*ver)->particles_in_size() >= in.size())
      if ((*ver)->particles_out_size() >= out.size()){

        //Check on incoming particles: make a tick-off list
        std::vector<Int_t> inl = in;

        //Loop over parents
        GenVertex::particle_iterator
          par = (*ver)->particles_begin(HepMC::parents);
        for (; par != (*ver)->particles_end(HepMC::parents); ++par){
          //Loop over requested particles to see if its there
          std::vector<Int_t>::iterator ireq_in = inl.begin();
          while (ireq_in != inl.end()){
            //Remove from whishlist if found
            if ((*ireq_in) == (*par)->pdg_id()){
              inl.erase(ireq_in);
              break ;
            }
            //goto next
            ireq_in++;
          }

          //Once all in particles are found
          if (inl.size() == 0) { 

            //Check on outcoming particles: make a tick-off list
            std::vector<Int_t> outl = out;

            //Loop over children
            GenVertex::particle_iterator
              par = (*ver)->particles_begin(HepMC::children);
            for (; par != (*ver)->particles_end(HepMC::children); ++par){
              //Loop over requested particles to see if its there
              std::vector<Int_t>::iterator ireq_out = outl.begin();
              while (ireq_out != outl.end()){
                //Remove from whishlist if found
                if ((*ireq_out) == (*par)->pdg_id()){
                  outl.erase(ireq_out);
                  break ;
                }
                //goto next
                ireq_out++;
              }

              //If also all out particles are found
              if (outl.size() == 0) {
                //Set this as a signal process vertex
                event.set_signal_process_vertex(*ver);
                return kTRUE;
              } //if children found
            } //for children
          } //if parents found
        } //for parents
      } //if enough particles
  } //for vertices
  
  return kFALSE;
}

void VisualFilter::RemoveGeant( HepMC::GenEvent& event ){

  Int_t MinGeantBarcode = 200000 ;
  Int_t MaxGeantBarcode = 999999 ;
  Int_t nRemoved=0;
  Int_t nRemovedP=0;

  //Remove vertices first
  for (GenEvent::vertex_iterator ver = event.vertices_begin();
       ver != event.vertices_end(); ver++)
    //Check for Geant barcode
    if ((abs((*ver)->barcode()) >= MinGeantBarcode) &&
        (abs((*ver)->barcode()) <= MaxGeantBarcode))
      //remove the vertex
      if ( event.remove_vertex(*ver) ) nRemoved++;
  cout << "Removed " << nRemoved << " GEANT vertices " << endl;

  //Now remove remaining particles
  for (GenEvent::particle_iterator par = event.particles_begin();
       par != event.particles_end(); par++)
    //Check for Geant barcode
    if (abs((*par)->barcode()) > MinGeantBarcode){
      //remove the particles from both its vertices
      if ( (*par)->production_vertex() ) 
        (*par)->production_vertex()->remove_particle(*par);
      if ( (*par)->end_vertex() ) 
        (*par)->end_vertex()->remove_particle(*par);
      nRemovedP++;
    }
  cout << "Removed " << nRemovedP << " GEANT particles " << endl;
}

void VisualFilter::RemovePythiaFragmentation(HepMC::GenEvent& event){

  //According to Pythia 6.4 manual, the Pythia internal particle id codes are
  // 91 - cluster : Parton system in cluster fragmentation
  // 92 - string : Parton system in string fragmentation
  // 93 - indep. : Parton system in independent fragmentation
  // 94 - CMshower : Four-momentum of time-like showering system
  // -- we remove only the these four. For a complete list see page 53 of the 
  // Pythia 6.4 manual (hep-ph/0603175)

  Int_t IDs[] = { 91, 92, 93, 94 };
  std::vector<int> FragIDs(IDs, IDs+4); 

  //remember how many particles and vertices we have removed
  Int_t nRemovedV=0;
  Int_t nRemovedP=0;

  //Loop over particles
  for (GenEvent::particle_iterator par = event.particles_begin();
       par != event.particles_end(); ++par){
    //Check for Hadronisation PDG Id 
    std::vector<int>::iterator res; 
    res = std::find( FragIDs.begin(), FragIDs.end(), (*par)->pdg_id());
    if ( res != FragIDs.end()){
      GenVertex* pv = (*par)->production_vertex();
      //Detach all in-particles as well
      if (pv)
        for (GenVertex::particle_iterator par =
          pv->particles_begin(HepMC::parents); par !=
          pv->particles_end(HepMC::parents); par++){
          pv->remove_particle(*par);
          nRemovedP++;
        }
      //remove vertex
      if (event.remove_vertex(pv)) nRemovedV++;
//      if(ev->remove_particle(*par)) nRemovedP++;
//      //Break at production-vertex
//      if ( remove_vertex((*par)->production_vertex()) ) nRemoved++;
//        else cout << "Not removed: "; (*par)->print();
    }
  }

  cout << "Removed " << nRemovedV << " fragmentation vertices " << endl;
  cout << "Removed " << nRemovedP << " fragmentation particles " << endl;
}
