//-- standard headers
#include <stdlib.h>
#include <stdio.h>

//-- ROOT headers
#include <TROOT.h>
#include <TApplication.h>
#include <TCanvas.h>
#include <TSystem.h>
#include <TButton.h>
#include <Riostream.h>

//-- HepMC and HepMCVisuals headers
#include "HepMC/SimpleVector.h"
#include "HepMC/GenEvent.h"
#include "../HepMCVisual/HepMCVisualElements.h"
#include "../HepMCVisual/HepMCVisualFilter.h"

//-- code that builds the individual events
#include "events/HerwigEvent0.C"
#include "events/HerwigEvent1.C"
#include "events/HerwigEvent2.C"
#include "events/HerwigEvent3.C"
#include "events/HerwigEvent4.C"
#include "events/HerwigEvent5.C"
#include "events/HerwigEvent6.C"
//#include "events/HerwigEvent7.C"
//#include "events/HerwigEvent8.C"
//#include "events/HerwigEvent9.C"

#include "events/PythiaEvent0.C"
#include "events/PythiaEvent1.C"
#include "events/PythiaEvent2.C"
#include "events/PythiaEvent3.C"
#include "events/PythiaEvent4.C"
#include "events/PythiaEvent5.C"
#include "events/PythiaEvent6.C"
//#include "events/PythiaEvent7.C"
//#include "events/PythiaEvent8.C"
//#include "events/PythiaEvent9.C"

//print usage message
void usage(char* progname){
  cout << "Usage: " << progname << " --pythia | --herwig [--remove-fragmentation]" << endl;
}


//main routine
int main( int argc, char** argv ){

   //Check command line arguments
   Bool_t RemoveFrag = false;
   Bool_t ShowHerwig = false;
   Bool_t ShowPythia = false;

   //check proper number of arguments
   if (argc < 2 || argc > 3){
     usage(argv[0]);
     return 1;
   }
   
   //check for data type
   if (TString(argv[1]).Contains("pythia")) ShowPythia=true;
   if (TString(argv[1]).Contains("herwig")) ShowHerwig=true;
   if (! (ShowHerwig || ShowPythia)){
     usage(argv[0]);
     return 1;
   }

   //check for fragmentation removal flag
   if (argc==3){
     if (TString(argv[2]) != TString("--remove-fragmentation")){
       usage(argv[0]);
       return 1;
     } else
        RemoveFrag=true;
   }
      
   TApplication theApp("App", &argc, argv);
 
   if (gROOT->IsBatch()) {
      fprintf(stderr, "%s: cannot run in batch mode\n", argv[0]);
      return 1;
   }
 
   //now generate the MonteCarlo events
   HepMC::GenEvent* Evts[20]; 
   int first=-1;
   int last =-1;
   
   if (ShowPythia) {
     Evts[0]= MakePEvent0();
     Evts[1]= MakePEvent1();
     Evts[2]= MakePEvent2();
     Evts[3]= MakePEvent3();
     Evts[4]= MakePEvent4();
     Evts[5]= MakePEvent5();
     Evts[6]= MakePEvent6();
//     Evts[7]= MakePEvent7();
//     Evts[8]= MakePEvent8();
//     Evts[9]= MakePEvent9();
     first = 0; last =7;
   }

   if (ShowHerwig) {
     Evts[7]= MakeHEvent0();
     Evts[8]= MakeHEvent1();
     Evts[9]= MakeHEvent2();
     Evts[10]= MakeHEvent3();
     Evts[11]= MakeHEvent4();
     Evts[12]= MakeHEvent5();
     Evts[13]= MakeHEvent6();
//     Evts[17]= MakeHEvent7();
//     Evts[18]= MakeHEvent8();
//     Evts[19]= MakeHEvent9();
     if (ShowPythia) { first = 0; last =14; }
       else { first = 7 ; last =14; }
   }

   //Add a filter to find the higgs vertex in Pythia events
   HepMC::VisualFilter* filter = new HepMC::VisualFilter();

   //Now loop over the events
   for (int i=first; i<last; i++){

     //Make a new Canvas and set the title
     TCanvas* CHepMC = new TCanvas("CHepMC","HepMCVisual Events",900,600);
     //Disable canvas menubar
     CHepMC->GetCanvasImp()->ShowMenuBar(kFALSE);

     //Convert Event in a visual event
     HepMC::VisualEvent* VEvt = HepMC::Visualize(Evts[i]);
     
     //Remove fragmentation if requested
     if (RemoveFrag)
       filter->RemovePythiaFragmentation(*VEvt);
     
     //Find signal process vertex for pythia events
     if ( i < 7 ){
       //Get a list of in and out particles at the signal vertex
       std::vector<Int_t> in;
       std::vector<Int_t> out;
       out.push_back(25); //PDG ID of higgs is 25
       //Search for the first vertex that has all these particles
       if (filter->FindSignalProcess(*VEvt, in, out))
         cout << Form("Event %i:",i) << "Found Higgs vertex" << endl;
     }
     //Draw the event
     VEvt->Draw();

     //Add a quit button
     TButton* quitb = new TButton("Quit","gApplication->Terminate()",
                                  .9,.08,.99,0.01);
     quitb->Draw();
     //Add a next event button
     TButton* nextb = new TButton("Next","gPad->Close()",
                                  .01,.08,.08,.01);
     nextb->Draw();

     //Get a canvas window handle
     TGWindow* cwin = gClient->GetWindowById(gVirtualX->GetWindowID(CHepMC->GetCanvasID()));
     //Handle everything that happens in the canvas
     gClient->ProcessEventsFor(cwin);
     //Wait until canvas window is closed
     gClient->WaitForUnmap(cwin);
     //clean up
     VEvt->Delete();
   }

   return 0;
}

